-----------------
-- Declaration --
-----------------
Yata = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0", "AceDebug-2.0", "CandyBar-2.0");
local BS = AceLibrary("Babble-Spell-2.2");
local dewdrop = AceLibrary("Dewdrop-2.0");
local deformat = AceLibrary("Deformat-2.0");
local YataTT = AceLibrary("Gratuity-2.0");

local L = AceLibrary("AceLocale-2.2"):new("Yata");
L:RegisterTranslations("enUS", function() 
	return {
	["Air"] = true,
	["Water"] = true,
	["Earth"] = true,
	["Fire"] = true,
	["Totem"] = true,
	["totem"] = "Totem",
	["Tools"] = true,
	["health"] = true,
	["points"] = true,
	["min"] = true,
	["sec"] = true,
	["Button Lock"] = true,
	["Lock the frame"] = true,
	["Button Scale"] = true,
	["Scale the buttons"] = true,
	["Bar Scale"] = true,
	["Scale the bars"] = true,
	["Reset"] = true,
	["Bar Lock"] = true,
	["Cooldown bars"] = true,
	["Totemic Call"] = true,
	};
end);
L:RegisterTranslations("deDE", function() 
	return {
	["Air"] = "Luft",
	["Water"] = "Wasser",
	["Earth"] = "Erde",
	["Fire"] = "Feuer",
	["Totem"] = "Totem",
	["totem"] = "totem",
	["Tools"] = "Werkzeuge",
	["health"] = "Gesundheit",
	["points"] = "Punkt%(en%)",
	["min"] = "Min.",
	["sec"] = "Sek.",
	["Button Lock"] = "Knpfe sperren",
	["Lock the frame"] = "Fenster sperren",
	["Button Scale"] = "Knopf Skalierung",
	["Scale the buttons"] = "Knpfe skalieren",
	["Bar Scale"] = "Balken Skalierung",
	["Scale the bars"] = "Balken skalieren",
	["Reset"] = "Zurcksetzen",
	["Bar Lock"] = "Balken sperren",
	["Cooldown bars"] = "Rahmen fr Cooldownbalken",
	["Totemic Call"] = "Ruf des Totems";
	};
end);

local defaults = {
	Locked = true,
	ButtonScale = 1.0,
	BarScale = 1.0;
	ButtonLocked = true,
	BarLocked = true,
	ShowTooltip = true,
	ShowTTOOC = true,
	ActiveTotemset = "Default",
	CDBarEnabled = true,
	InvertBar = false,
};
Yata:RegisterDB("YataDB");
Yata:RegisterDefaults('profile', defaults);
local Yata_TotemTiming = {L["min"], L["sec"]};
-- potential Blacklist to get around spells with the Totemstring and which aren't Totems at all
local Yata_SpellBlacklist = {L["Totemic Call"]};

----------------
-- Code below --
----------------
function Yata:OnInitialize()
	self.options = {
		type="group",
		args = {
			addonheader = {
				name = "Yata",
				type = "header",
				order = 1,
			},
			lock = {
				name = L["Lock the frame"],
				desc = L["Lock the frame"],
				type = "toggle",
				get = function() return self.db.profile.Locked end,
				set = function(v)
					self.db.profile.Locked = v;
					if(self.IsInitialized) then
						self:SaveConfig();
					end
				end,
				disabled = function() return InCombatLockdown() end,
			},
			buttonlock = {
				name = L["Button Lock"],
				type = "toggle",
				desc = L["Button Lock"],
				get = function() return self.db.profile.ButtonLocked end,
				set = function(v)
					self.db.profile.ButtonLocked = v;
					if(self.IsInitialized) then
						self:SetupSwap(v);
						self:SaveConfig();
					end
				end,
				disabled = function() return InCombatLockdown() end,
			},
			barlock = {
				name = L["Bar Lock"],
				desc = L["Bar Lock"],
				type = "toggle",
				get = function() return self.db.profile.BarLocked end,
				set = function(v)
					self.db.profile.BarLocked = v;
					if(self.IsInitialized) then
						self:SetupBarMovement(v);
						self:SaveConfig();
					end;
				end,
				disabled = function() return InCombatLockdown() end,
			},
			scale = {
				name = L["Button Scale"],
				desc = L["Scale the buttons"],
				type = "range",
				get = function() return self.db.profile.ButtonScale end,
				set = function(v)
					self.db.profile.ButtonScale = v;
					if(self.IsInitialized) then
						self:SaveConfig();
					end
				end,
				min = 0.5,
				max = 1.5,
				step = 0.05,
				disabled = function() return InCombatLockdown() end,
			},
			barscale = {
				name = L["Bar Scale"],
				desc = L["Scale the bars"],
				type = "range",
				get = function() return self.db.profile.BarScale end,
				set = function(v)
					self.db.profile.BarScale = v;
					if(self.IsInitialized) then
						self:SaveConfig();
					end
				end,
				min = 0.5,
				max = 1.5,
				step = 0.05,
				disabled = function() return InCombatLockdown() end,
			},
			cdbarenable = {
				name = "Enable Cooldown bar",
				desc = "Enable Cooldown bar",
				type = "toggle",
				get = function() return self.db.profile.CDBarEnabled end,
				set = function(v)
					self.db.profile.CDBarEnabled = v;
					if(self.IsInitialized) then
						self:SaveConfig();
					end
				end,
				disabled = function() return InCombatLockdown() end,
			},
			invertbar = {
				name = "Invert bar direction",
				desc = "Invert bar direction",
				type = "toggle",
				get = function() return self.db.profile.InvertBar end,
				set = function(v)
					self.db.profile.InvertBar = v;
					if(self.IsInitialized) then
						self:Print("You'll need to /console reloadui now");
						self:SaveConfig();
					end
				end,
				disabled = function() return InCombatLockdown() end,
				},
			tooltips = {
				name = "Tooltips",
				desc = "Tooltip configuration",
				type = "group",
				args = {
					showtooltip = {
						name = "Show Tooltip",
						type = "toggle",
						desc = "Show Tooltip",
						get = function() return self.db.profile.ShowTooltip end,
						set = function(v)
							self.db.profile.ShowTooltip = v;
							self.db.profile.ShowTTOOC = v;
							if(self.IsInitialized) then
								self:SaveConfig();
							end
						end,
					},
					showooconly = {
						name = "Out of combat only",
						type = "toggle",
						desc = "Show Tooltip out of combat only",
						get = function() return self.db.profile.ShowTTOOC end,
						set = function(v)
							self.db.profile.ShowTTOOC = v;
							if(self.IsInitialized) then
								self:SaveConfig();
							end
						end,
					},			
				},
				disabled = function() return InCombatLockdown() end,
			},
			totemsets = {
				name = "Totemsets",
				desc = "Totemsets",
				type = "group",
				args = {
					addset = {
						name = "Add Set",
						desc = "Add Set",
						type = "execute",
						func = function(name) Yata:AddTotemset("hallo") end,
					},
					useset = {
						name = "Use Set",
						desc = "Use Set",
						type = "group",
						args = {},
					},
				},
				disabled = function() return InCombatLockdown() end,
			},
			keybindings = {
				name = "Keybindings",
				desc = "Keybindings",
				type = "group",
				args = {},
				disabled = function() return InCombatLockdown() end,
			},
			reset = {
				name = L["Reset"];
				desc = L["Reset"];
				type = "execute",
				func = "Reset",
				order = 101,
				disabled = function() return InCombatLockdown() end,
			},
		}
	};
	self.formatStrings = {
		SPELLLOGCRITOTHEROTHER,
		SPELLLOGCRITSCHOOLOTHEROTHER,
		SPELLLOGOTHEROTHER,
		SPELLLOGSCHOOLOTHEROTHER,
		COMBATHITCRITOTHEROTHER,
		COMBATHITOTHEROTHER,
	};
	self:RegisterChatCommand({"/yata"}, self.options );
end
-----------------
-- Disable Mod --
-----------------

function Yata:OnDisable()
	self:UnhookAll();
	self:UnregisterAllEvents();
	for k,b in pairs(self.Buttons) do
		b.Frame:Hide();
		b = nil;
	end
	self.Buttons = nil;
	self.TotemData = nil;
	self.IsInitialized = false;
end
----------------
-- Enable Mod --
----------------

function Yata:OnEnable()
	self.IsInitialized = false;
	local _, EnglishClass = UnitClass( "player" );
	if( EnglishClass ~= "SHAMAN" ) then return end;
	self:RegisterEvent("PLAYER_DEAD");
	-- Watch for damage done to our totems
	self:RegisterEvent("CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE", "TotemDamage");
	self:RegisterEvent("CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS", "TotemDamage");
	self:RegisterEvent("CHAT_MSG_SPELL_HOSTILEPLAYER_DAMAGE", "TotemDamage");
	self:RegisterEvent("CHAT_MSG_COMBAT_HOSTILEPLAYER_HITS", "TotemDamage");
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
	self:RegisterCandyBarGroup("YataCandy");
	self:SetCandyBarGroupGrowth("YataCandy",false);
	self.ElementsCount = {["Earth"] = 0, ["Air"] = 0, ["Water"] = 0, ["Fire"] = 0};
	if(not self.db.profile.TotemsOrder) then
		self.db.profile.TotemsOrder = {"Earth","Air","Water","Fire"};
		for i=1,4 do
			self.db.profile.TotemsOrder[self.db.profile.TotemsOrder[i]] = {};
		end
	end
	if(not self.db.profile.Keybindings) then
		self.db.profile.Keybindings = {};
	end
	self:GatherTotemData();
	if(self.TotemData.Count ~= self.db.profile.TotemsCount) then
		self:Print("Totem count mismatch: resetting...");
		self.db.profile.TotemsOrder = {"Earth","Air","Water","Fire"};
		for i=1,4 do
			self.db.profile.TotemsOrder[self.db.profile.TotemsOrder[i]] = {};
		end
	end
	if(not self.db.profile.Totemsets) then
		self.db.profile.Totemsets = {};
		self.db.profile.Totemsets["Default"] = {};
		for i=1,4 do
			(self.db.profile.Totemsets["Default"])[self.db.profile.TotemsOrder[i]] = {};
		end
	end
	self:InitFrame();
	self:LoadPosition();
	for k,b in pairs(self.Buttons) do
		self:CreateTotemKeybindingOption(b.ButtonID);
		local hotkey = getglobal( b.Frame:GetName() .. "HotKey" );
		hotkey:SetText(GetBindingText(self.db.profile.Keybindings[b.ButtonID], "KEY_"));
	end
	for k,s in pairs(self.db.profile.Totemsets) do
		self:CreateTotemSetOptionEntry(k);
	end
	dewdrop:Register(Minimap,'children', function() dewdrop:FeedAceOptionsTable(self.options) end);
	-- Ugly Hooks below --
	hooksecurefunc("SecureStateAnchor_RunChild",function(self,button,remap) Yata:ControlOnEnter(self,button) end);
	----------------------
	self.IsInitialized = true;
end
-----------------
-- Event Stuff --
-----------------

function Yata:PLAYER_DEAD()
	--- todo
end
-- TotemDamage function from GotWood by MoonSorrow
function Yata:TotemDamage(msg)
	if not string.find(string.lower(msg), string.lower(L["Totem"])) then return end
	local totem, dmg
	for _,v in ipairs(self.formatStrings) do
		local arg1, arg2, arg3, arg4 = deformat(msg, v)
		if arg4 and type(arg4)=='number' then
			totem, dmg = arg3, arg4
			break
		elseif arg2 and type(arg3)=='number' then
			totem, dmg = arg2, arg3
			break
		end
	end
	if totem and dmg then
		for k,b in pairs(self.Buttons) do
			if(string.find(totem,b.Totem.Name) and b.CurrentHealth) then
				b.CurrentHealth = b.CurrentHealth - dmg;
				if(b.CurrentHealth <= 0) then
					self:StopTimer(b.ButtonID);
				end
			break;
			end
		end
	end
end

function Yata:UNIT_SPELLCAST_SUCCEEDED(player, SpellName, sRank)
	if (player == "player") then 
		self:CheckHook(SpellName);
	end
end
--------------------
-- Frames & Stuff --
--------------------
function Yata:UpdateScale()
	for k,b in pairs(self.Buttons) do
		if(b.ButtonID < 5) then
			b.Frame:SetScale(self.db.profile.ButtonScale);
		else b.Frame:SetScale(self.db.profile.ButtonScale - 0.2);
		end
	end
	self.CandyAnchorFrame:SetScale(self.db.profile.BarScale);
end
function Yata:ReturnOrderByElement(ele)
	local retval = 0;
	for i=1,4 do
		if(self.db.profile.TotemsOrder[i] == ele) then
			retval = i;
			break;
		end
	end
	return retval;
end



-- Buttons: 1-4 are the Control Buttons, 11-1X are the MouseOver Buttons for Element 1, 21-2X for Element 2 etc.
function Yata:InitFrame()
	self.Buttons = {};			
	-- First we create the Header
	self:CreateHeader();
	-- After the header, we generate 4 Control buttons
	for i = 1,4 do 
		self.Buttons[i] = self:CreateControlButton(i,self.db.profile.TotemsOrder[i].."CONTROL");
	end
	-- Then we populate them and create other buttons on the fly
	-- or we load config data
	for k,v in ipairs(self.TotemData) do
		local totem = self.TotemData[v];
		totem = totem[table.getn(totem)];
		local number = 0;
		local pos = self:GetPosByTotem(totem);
		if(pos == 1) then
			number = pos * self:ReturnOrderByElement(totem.Element);
		elseif(pos > 1) then
			number = pos - 1 + self:ReturnOrderByElement(totem.Element)*10;
		elseif(pos == 0) then
			pos = self:ReturnOrderByElement(totem.Element);
			if(not self.Buttons[pos].Totem) then
				number = pos;
			else
				number = pos*10 + self.ElementsCount[totem.Element];
			end
		end
		if(not self.Buttons[number]) then
			self.Buttons[number] = self:CreateButton(number,totem.Element);
		end
		self.Buttons[number] = self:PopulateButton(self.Buttons[number],totem);
		self.ElementsCount[totem.Element] = self.ElementsCount[totem.Element] + 1;
	end
end
function Yata:GetPosByTotem(totem)
	local retval = 0;
	if(self.db.profile.TotemsOrder[totem.Element]) then
		for k,v in pairs(self.db.profile.TotemsOrder[totem.Element]) do
			if(v == totem.Name) then retval = k;
				break;
			end
		end
	end
	return retval;
end
function Yata:CreateHeader()
	self.Header = CreateFrame("Frame", "YATAHEADERFRAME",  UIParent, "SecureStateDriverTemplate");
	self.Header:SetAttribute("statemap-Totem", "$input");
	self.Header:SetWidth(10);
	self.Header:SetHeight(10);
	for i = 1, 4 do
		self.Header:SetAttribute("statemap-anchor-enter"..i, "*:"..i);
		self.Header:SetAttribute("statemap-anchor-leave"..i,      ";");
		self.Header:SetAttribute("delaystatemap-anchor-leave"..i, i..":0");
		self.Header:SetAttribute("delaytimemap-anchor-leave"..i,  i..":0.5");
		self.Header:SetAttribute("delayhovermap-anchor-leave"..i, i..":true");
	end
	self.Header:Show();
	self.Header:RegisterForDrag( "LeftButton" );
end
function Yata:CreateControlButton(ID,ele)
	local b = {};
	b.Frame = getglobal("YATABUTTON"..string.upper(ele)..ID);
	if(not b.Frame) then
		b.Frame = CreateFrame("Button", "YATABUTTON"..string.upper(ele)..ID, self.Header, "ActionButtonTemplate,SecureActionButtonTemplate,SecureAnchorEnterTemplate");
		b.ButtonID = ID;
	end
	self.Header:SetAttribute("AddChild",b.Frame);
	b.Time = b.Frame:CreateFontString("$parentTime","ARTWORK","GameFontNormalLarge");
	b.Time:SetPoint("CENTER", b.Frame, "CENTER", 0, -5);
	b.Time:SetVertexColor(1,1,1,1);
	b.Frame:SetScale(self.db.profile.ButtonScale);
	b.Frame:SetAttribute("showstates", "*");
	b.Frame:SetAttribute("delaystate", "0");
    	b.Frame:SetAttribute("delaytime", "0.5");
    	b.Frame:SetAttribute("delayhover", "any");
    	b.Frame:SetAttribute("newstate", tostring(ID));
    	b.Frame:SetAttribute("*childraise-OnEnter", true);
    	b.Frame:SetAttribute("*childstate-OnEnter", "enter"..ID);
    	b.Frame:SetAttribute("*childstate-OnLeave", "leave"..ID);
	b.Frame:SetAttribute("anchorchild", self.Header);
    	b.Frame:SetAttribute("childstate", "^" .. tostring(ID));
    	b.Frame:RegisterForClicks("LeftButtonUp","RightButtonUp");
    	b.Frame:SetScript("PostClick",function() Yata:OnButtonClick(ID) end);
    	b.Frame:Show();
	return b;
end
function Yata:CreateButton(ID,ele)
	local b = {};
	b.Frame = getglobal("YATABUTTON"..string.upper(ele)..ID);
	if(not b.Frame) then
		b.Frame = CreateFrame("Button", "YATABUTTON"..string.upper(ele)..ID, self.Header, "SecureActionButtonTemplate,ActionButtonTemplate");
	end
	b.ButtonID = ID;
	b.Time = b.Frame:CreateFontString("$parentTime","ARTWORK","GameFontNormalLarge");
	b.Time:SetPoint("CENTER", b.Frame, "CENTER", 0, -5);
	b.Time:SetVertexColor(1,1,1,1);
	b.Frame:SetScale(self.db.profile.ButtonScale);
	b.Frame:SetAttribute("showstates", math.floor(ID/10));
	b.Frame:SetAttribute("newstate", "0");
	b.Frame:RegisterForClicks("LeftButtonUp","RightButtonUp");
	b.Frame:SetScript("PostClick",function() Yata:OnButtonClick(ID) end);
	b.Frame:SetScript("OnEnter", function() if(Yata:CheckShowToolTip()) then Yata:SetTooltip(ID) end end)
	b.Frame:SetScript("OnLeave", function() if(Yata:CheckShowToolTip()) then GameTooltip:Hide(); end end)
	self.Header:SetAttribute("AddChild",b.Frame);
	return b;
end

function Yata:PopulateButton(b, totem)
	local btexture = getglobal(b.Frame:GetName().."Icon");
	btexture:SetTexture(totem.Texture);
	b.Totem = totem;
	b.Frame:SetAttribute("type*", "spell");
	b.Frame:SetAttribute("spell1", b.Totem.Name);
	b.Frame:SetAttribute("spell2", L["Totemic Call"]);
	b.Frame:SetAttribute("shift-type1", "ATTRIBUTE_NOOP");
	return b;
end
function Yata:HeaderMovement()
	local b = self.Buttons[1];
	b.Frame:SetMovable(true);
	b.Frame:SetScript("OnMouseDown",function()
				if (arg1 == "LeftButton" and not self.db.profile.Locked) then
					this:StartMoving();
					end
				end);
	b.Frame:SetScript("OnMouseUp",function()
				if (arg1 == "LeftButton") then
					this:StopMovingOrSizing();
					self:SaveConfig();
					end
				end);
	b.Frame:SetScript("OnHide",function() this:StopMovingOrSizing() end)
end

function Yata:CandyAnchor()
	self.CandyAnchorFrame = {};
	self.CandyAnchorFrame = CreateFrame("Frame", "YATACANDYANCHOR", UIParent);
	self.CandyAnchorFrame:Hide();
	self.CandyAnchorFrame:SetWidth(250);
	self.CandyAnchorFrame:SetHeight(100);
	self.CandyAnchorFrame:SetBackdrop({bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", tile = true, tileSize = 16,
					edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
					edgeSize = 16, insets = {left = 4, right = 4, top = 4, bottom = 4}});
	self.CandyAnchorFrame:SetBackdropBorderColor(.5, .5, .5);
	self.CandyAnchorFrame:SetBackdropColor(0,0,0);
	self.CandyAnchorFrame.Text = self.CandyAnchorFrame:CreateFontString("$parentTime","ARTWORK","GameFontNormalLarge");
	self.CandyAnchorFrame.Text:SetPoint("CENTER", self.CandyAnchorFrame, "CENTER", 0, 0);
	self.CandyAnchorFrame.Text:SetVertexColor(1,1,1,1);
	self.CandyAnchorFrame.Text:SetText(L["Cooldown bars"]);
	self.CandyAnchorFrame:EnableMouse(true);
	self.CandyAnchorFrame:RegisterForDrag("LeftButton");
	self.CandyAnchorFrame:SetMovable(true);
	self.CandyAnchorFrame:SetScript("OnDragStart", function() this:StartMoving(); end);
	self.CandyAnchorFrame:SetScript("OnDragStop", function() this:StopMovingOrSizing(); self:SaveConfig(); end);
	self.CandyAnchorFrame:SetScale(self.db.profile.BarScale);
	self.CandyAnchorFrame:SetPoint( "CENTER", WorldFrame, "CENTER", 0,0 );
	
	local x = 0;
	local y = 0;
	local scale = 1;
	if(self.db.profile.CooldownPosition) then
		x = self.db.profile.CooldownPosition.x;
		y = self.db.profile.CooldownPosition.y;
		scale = self.CandyAnchorFrame:GetEffectiveScale();
	else
		self.CandyAnchorFrame:ClearAllPoints();
		self.CandyAnchorFrame:SetPoint( "CENTER", WorldFrame, "CENTER", 0,0 );
		scale = self.CandyAnchorFrame:GetEffectiveScale();
		local worldscale = UIParent:GetEffectiveScale();
		x = self.CandyAnchorFrame:GetLeft()*scale;
		y = self.CandyAnchorFrame:GetTop()*scale - (UIParent:GetTop())*worldscale;
	end
	self.CandyAnchorFrame:SetPoint("TOPLEFT", UIParent,"TOPLEFT", x/scale, y/scale);
	self:SetCandyBarGroupPoint("YataCandy","CENTER",self.CandyAnchorFrame,"CENTER",0,25*self.db.profile.BarScale);
end
-----------
-- Hooks --
-----------

-- If something triggers a potential Totemstomp we check for it and start the timer if necessary
function Yata:CheckHook(SpellName)
	local istotem = nil;
	if (string.find(string.lower(SpellName), string.lower(L["Totem"]))  and not self:SpellBlacklist(SpellName)) then
		istotem = true;
	elseif (SpellName == L["Totemic Call"]) then
		istotem = nil;
	else return end;
	local ID = nil;
	for k,b in pairs(self.Buttons) do
		if(istotem and string.find(string.lower(SpellName),string.lower(b.Totem.Name))) then
			if(self.db.profile.CDBarEnabled) then
				b.CurrentHealth = b.Totem.Health;
				self:StartTimer(b.ButtonID);
			end
			ID = b.ButtonID;
			break;
		elseif(b.TimeStart and not istotem) then
			if(self.db.profile.CDBarEnabled) then
				self:StopTimer(b.ButtonID);
			end
			ID = b.ButtonID;
			break;
		end
	end
	self:GlobalCooldown(ID);
end

function Yata:ControlOnEnter(button, type)
	if(not self:CheckShowToolTip() or not self.IsInitialized) then return end;
	if(type == "OnEnter") then
		local _, _, ID = string.find(button:GetName(),"%u+(%d+)");
		self:SetTooltip(tonumber(ID));
	elseif(type == "OnLeave") then
		GameTooltip:Hide();
	end
end
--------------
-- Tooltips --
--------------
function Yata:SetTooltip(ID)
	local frame = self.Buttons[ID].Frame;
	if ( GetCVar("UberTooltips") == "1" ) then
		GameTooltip_SetDefaultAnchor(GameTooltip, frame);
	else
		GameTooltip:SetOwner(frame, "ANCHOR_RIGHT");
	end
	GameTooltip:SetSpell( self.Buttons[ID].Totem.SpellID, BOOKTYPE_SPELL );
end

function Yata:CheckShowToolTip()
	if(not self.db.profile.ShowTooltip) then return nil end;
	if(self.db.profile.ShowTooltip and self.db.profile.ShowTTOOC and InCombatLockdown()) then return nil end;
	return true;
end

----------------------
-- On Button Events --
----------------------

function Yata:SetupBarMovement(locked)
	if(not locked) then
		self.CandyAnchorFrame:Show();
	else 
		self.CandyAnchorFrame:Hide();
	end

end



function Yata:SetupSwap(locked)
	if(not locked) then
		for k,b in pairs(self.Buttons) do
			b.Frame:SetAttribute("showstates", "*");
			b.Frame:Show();
		end
	else 
		for k,b in pairs(self.Buttons) do
			if(b.ButtonID > 10) then
				b.Frame:SetAttribute("showstates", math.floor(b.ButtonID/10));
				b.Frame:Hide();
			end
		end
	end

end

function Yata:OnButtonClick(ID)
	if(self.db.profile.Locked and not self.db.profile.ButtonLocked) then
		if(not self.StartSwapButton) then
			self.Buttons[ID].Time:SetText("S1");
			self.StartSwapButton = ID;
		else
			self.StopSwapButton = ID;
			self.Buttons[ID].Time:SetText("S2");
			Yata:SwapButton(self.StartSwapButton, self.StopSwapButton);
			self.Buttons[self.StartSwapButton].Time:SetText("");
			self.Buttons[self.StopSwapButton].Time:SetText("");
			self.StartSwapButton = nil;
			self.StopSwapButton = nil;
		end
	end
end
function Yata:SwapButton(start, stop)
	-- Swapping buttons with the same element is easy
	local s1 = self.Buttons[start].Totem;
	local s2 = self.Buttons[stop].Totem;
	local s1ele = s1.Element;
	local s2ele = s2.Element;
	if(s1ele == s2ele) then
		-- Same Element
		self.Buttons[start] = self:PopulateButton(self.Buttons[start],s2);
		self.Buttons[stop] = self:PopulateButton(self.Buttons[stop],s1);
	-- The more complicated part aka swapping whole elements is not yet implemented 
	elseif(s1ele ~= s2ele and start < 5 and stop < 5) then
		self:Print("Please edit the savedvariables for now");
	end
end

----------------
-- Totem Sets --
----------------
function Yata:CreateTotemSetOptionEntry(name)
	local options = self.options.args.totemsets.args.useset.args;
	options[name] = {
		name = name;
		desc = "Load "..name;
		type = "execute",
		func = function(name) Yata:LoadTotemset(name) end,
	}

end
function Yata:AddTotemset(name)
	
end
function Yata:LoadTotemset(name)
	
end
-----------------
-- Keybindings --
----------------

-- Binding functions basicly from ClosetGnome by rabbit
function Yata:ClearBinding(ID)
	if not button then return end
	local oldbind = self.db.profile.Keybindings[ID];
	if not oldbind then return end
	self.db.profile.Keybindings[ID] = nil;
	SetBinding(oldbind);
	local hotkey = getglobal( self.Buttons[ID].Frame:GetName() .. "HotKey" );
	hotkey:SetText("");
end

function Yata:RegisterBinding(key, ID)
	if not key or not button then return end
	if not AceLibrary("AceConsole-2.0").keybindingValidateFunc(key) then
		self:Print(string.format("%s is not a valid keybinding.", "|cffd9d919"..key.."|r"))
		return
	end
	for k, v in pairs(self.db.profile.Keybindings) do
		if v == key and k ~= ID then
			self.db.profile.Keybindings[k] = nil;
		elseif k == ID and v ~= key then
			self.db.profile.Keybindings[k] = nil;
			SetBinding(v) -- remove old binding
		end
	end
	self.db.profile.Keybindings[ID] = key;
	SetBindingClick(key, self.Buttons[ID].Frame:GetName());
	local hotkey = getglobal( self.Buttons[ID].Frame:GetName() .. "HotKey" );
	hotkey:SetText(GetBindingText(key, "KEY_"));
	-- save our new bindings
	SaveBindings(GetCurrentBindingSet());
end
-- basically from Bartender3
function Yata:CreateTotemKeybindingOption(ID)
	local options = self.options.args.keybindings.args;
	local text = ""; 
	local shorttext = "";
	if(ID < 5) then
		text = Yata.Buttons[ID].Totem.Element.." Main Button ";
		shorttext = Yata.Buttons[ID].Totem.Element.."MainButton";
	else
		text = Yata.Buttons[ID].Totem.Element.." Menu Button "..math.fmod(ID,10);
		shorttext = Yata.Buttons[ID].Totem.Element.."MenuButton"..math.fmod(ID,10);
	end
	options[shorttext] = {
		type = "text",
		name = text,
		desc = "Assign a Keybinding to",
		validate = "keybinding",
		order = ID,
		get = function()
			return Yata.db.profile.Keybindings[ID];
		end,
		set = function(k)
			if k == nil then
				Yata:ClearBinding(ID);
			else
				Yata:RegisterBinding(k,ID);
			end
		end,
	}
end

--------------
-- Cooldown --
--------------

function Yata:StartTimer(ID)
	if(self:IsCandyBarRegistered(self.Buttons[ID].Totem.Element.."DurationBar")) then	
		-- if another totem with the same element is thrown --	
		for k,b in pairs(self.Buttons) do 
			if(b.TimeStart and b.Totem.Element == self.Buttons[ID].Totem.Element) then
				self:StopTimer(b.ButtonID);
				break;
			end
		end
	end
	self.Buttons[ID].TimeStart = time();
	self:RegisterCandyBar(self.Buttons[ID].Totem.Element.."DurationBar",self.Buttons[ID].Totem.Duration,self.Buttons[ID].Totem.Name,self.Buttons[ID].Totem.Texture,"green","yellow","orange","red");
	self:RegisterCandyBarWithGroup(self.Buttons[ID].Totem.Element.."DurationBar", "YataCandy");
	self:SetCandyBarCompletion(self.Buttons[ID].Totem.Element.."DurationBar", function(ID) Yata:StopTimer(ID) end, ID) 		
	self:StartCandyBar(self.Buttons[ID].Totem.Element.."DurationBar", false);
	self:SetCandyBarScale(self.Buttons[ID].Totem.Element.."DurationBar",self.db.profile.BarScale);
end
function Yata:StopTimer(ID)
	-- weird error somewhere, invalid key to next()
	--self:UnregisterCandyBarWithGroup(self.Buttons[ID].Totem.Element.."DurationBar", "YataCandy");
	--self:UnregisterCandyBar(self.Buttons[ID].Totem.Element.."DurationBar");
	self:StopCandyBar(self.Buttons[ID].Totem.Element.."DurationBar");
	if(self.Buttons[ID].Totem.Cooldown > 0) then
		local count = time() - self.Buttons[ID].TimeStart;
		if(count < self.Buttons[ID].Totem.Cooldown) then
			self:Cooldown(ID,self.Buttons[ID].Totem.Cooldown-count);
		end
	end
	self.Buttons[ID].TimeStart = nil;
end
function Yata:Cooldown(ID, count)
	self:RegisterCandyBar(self.Buttons[ID].Totem.Element.."CooldownBar",count,self.Buttons[ID].Totem.Name.." Cooldown",self.Buttons[ID].Totem.Texture,"red","orange","yellow","green");
	self:RegisterCandyBarWithGroup(self.Buttons[ID].Totem.Element.."CooldownBar", "YataCandy");
	self:StartCandyBar(self.Buttons[ID].Totem.Element.."CooldownBar",true);
	self:SetCandyBarScale(self.Buttons[ID].Totem.Element.."CooldownBar",self.db.profile.BarScale);
end
function Yata:GlobalCooldown(ID)
	start, duration, enable = GetSpellCooldown( self.Buttons[ID].Totem.SpellID, BOOKTYPE_SPELL );
	local now = GetTime();
	local castlag = now-start;
	for i=1,4 do
		local cooldown = getglobal( self.Buttons[i].Frame:GetName() .. "Cooldown" );
		CooldownFrame_SetTimer( cooldown, now, duration-castlag, enable );
	end
end
-----------------
-- Load & Save --
-----------------

function  Yata:LoadPosition()
	local x = 0;
	local y = 0;
	local scale = 1;
	if(self.db.profile.Position) then
		x = self.db.profile.Position.x;
		y = self.db.profile.Position.y;
		scale = self.Buttons[1].Frame:GetEffectiveScale();
	else
		self.Buttons[1].Frame:ClearAllPoints();
		self.Buttons[1].Frame:SetPoint( "CENTER", WorldFrame, "CENTER", 0,0 );
		scale = self.Buttons[1].Frame:GetEffectiveScale();
		local worldscale = UIParent:GetEffectiveScale();
		x = self.Buttons[1].Frame:GetLeft()*scale;
		y = self.Buttons[1].Frame:GetTop()*scale - (UIParent:GetTop())*worldscale;
	end
	local anchorTo;
	for k,b in pairs(self.Buttons) do
		b.Frame:ClearAllPoints();
		if(b.ButtonID == 1) then
		    b.Frame:SetPoint("TOPLEFT", UIParent,"TOPLEFT", x/scale, y/scale);
		    b.Frame:SetScale(self.db.profile.ButtonScale);
		elseif(b.ButtonID < 5) then
		    b.Frame:SetPoint("BOTTOMLEFT", anchorTo, "BOTTOMRIGHT", 5,0);
		    b.Frame:SetScale(self.db.profile.ButtonScale);
		else
		    if(math.fmod(b.ButtonID,10) == 1) then
		    	anchorTo = self.Buttons[math.floor(b.ButtonID/10)].Frame;
		    	if(self.db.profile.InvertBar) then
		    		b.Frame:SetPoint( "TOP", anchorTo, "BOTTOM", 0, -5);
		    	else
		    		b.Frame:SetPoint( "BOTTOM", anchorTo, "TOP", 0, 5);
		    	end
		    else
			anchorTo = self.Buttons[b.ButtonID-1].Frame;
			if(self.db.profile.InvertBar) then
				b.Frame:SetPoint( "TOPLEFT", anchorTo, "BOTTOMLEFT", 0, -5);
			else
				b.Frame:SetPoint( "BOTTOMLEFT", anchorTo, "TOPLEFT", 0, 5);
			end
		    end
		    b.Frame:SetScale(self.db.profile.ButtonScale-0.2);
		end
		anchorTo = b.Frame;
	end
	self:HeaderMovement();
	-- Create Candy Anchor
	self:CandyAnchor();
	self:SaveConfig();
	self.Header:SetPoint("TOPRIGHT", self.Buttons[1].Frame, "TOPLEFT",0,5);
	table.sort(self.Buttons,function(a,b) return a.ButtonID < b.ButtonID end);
end

function Yata:Reset()
	if(not InCombatLockdown()) then
		self.db.profile.Position = nil;
		self.Buttons[1].Frame:SetScale(1.0);
		self.CandyAnchorFrame:SetScale(1.0);
		self:LoadPosition();
	end
end
function Yata:ResetTotems()
	self:SaveConfig();
	self:GatherTotemData();
	self:InitFrame();
	self:LoadPosition();
end
function Yata:SaveConfig()
	if not self.db.profile.Position then 
		self.db.profile.Position = {};
	end
	local scale = self.Buttons[1].Frame:GetEffectiveScale();
	local worldscale = UIParent:GetEffectiveScale();
	local x = self.Buttons[1].Frame:GetLeft()*scale;
	local y = self.Buttons[1].Frame:GetTop()*scale - (UIParent:GetTop())*worldscale;
	self.db.profile.Position.x = x;
	self.db.profile.Position.y = y;
	
	if not self.db.profile.CooldownPosition then
		self.db.profile.CooldownPosition = {};
	end
	local scale = self.CandyAnchorFrame:GetEffectiveScale();
	local worldscale = UIParent:GetEffectiveScale();
	local x = self.CandyAnchorFrame:GetLeft()*scale;
	local y = self.CandyAnchorFrame:GetTop()*scale - (UIParent:GetTop())*worldscale;
	self.db.profile.CooldownPosition.x = x;
	self.db.profile.CooldownPosition.y = y;
	local count = 0;
	for i=1,4 do
		self.db.profile.TotemsOrder[self.db.profile.TotemsOrder[i]] = {};
	end
	for k,v in pairs(self.Buttons) do
		if(v.ButtonID < 5) then
			(self.db.profile.TotemsOrder[v.Totem.Element])[1] = v.Totem.Name;
			count = count + 1;
		else 
			(self.db.profile.TotemsOrder[v.Totem.Element])[math.fmod(v.ButtonID,10)+1] = v.Totem.Name;
			count = count + 1;
		end	
	end
	self.db.profile.TotemsCount = count;
	self:UpdateScale();
end

-----------------------------
-- Utility functions below --
-----------------------------
function Yata:SpellBlacklist(spell)
	local blacklist = nil;
	for k,v in pairs(Yata_SpellBlacklist) do
		if(spell == v) then
			blacklist = true;
			break;
		end
	end
	return blacklist
end

function Yata:GatherTotemData()
    self.TotemData = {};
    local tabs = GetNumSpellTabs();
    for tab = 2, tabs do
	local _, _, offset, numSpells = GetSpellTabInfo(tab);
	for i = 1, numSpells do
	    local spell = offset + i;
	    local SpellName, subspell = GetSpellName(spell, "spell");
	    if (string.find(string.lower(SpellName), string.lower(L["Totem"])) and not self:SpellBlacklist(SpellName)) then
		if (not self.TotemData[SpellName]) then
		    self.TotemData[SpellName] = {};
		    table.insert(self.TotemData, SpellName);
		end
		local data = {};
		data.SpellID = spell;
		table.insert(self.TotemData[SpellName],data);
	    end
	end
    end
    local count = 0;
    -- Add Spell Information (Cooldown,Duration, HP, Element)
    	for k,v in ipairs(self.TotemData) do
    		for l,w in ipairs(self.TotemData[v]) do
			local data = self:ExtractToolTip(w.SpellID);
			w.Element = data.element;
			w.Cooldown = data.Cooldown;
			w.Health = data.health;
			w.Duration = data.duration;
			w.Mana = data.mana;
			w.Name = v;
			w.Texture = GetSpellTexture(w.SpellID,"spell");
		end
		count = count + 1;
	end
    self.TotemData.Count = count;
end
--- Return Cooldown, Duration, HP, Element
--- Not yet fully converted to Gratuity/Deformat
function Yata:ExtractToolTip(spellid)
	local element = "";
	local data = {};
	data.element = "";
	data.Cooldown = 0;
	data.health = 0;
	data.duration = 0;
	data.mana = 0;
	YataTT:SetSpell( spellid, "spell" );
	-- Mana cost --
	data.mana = YataTT:FindDeformat(MANA_COST,2,2,false,true);
	-- Element extraction --
	local elesearch = L["Tools"]..": (.*)"..L["totem"];
	local _,_, element = YataTT:Find(elesearch,4,4,false,true);
	if( element ) then	
		if( string.sub( element, 1, 1 ) == "|" ) then
			element = string.sub( element, 11 );
		end
		_, _, element = string.find(element, "(%a*)");
		-- localization hacks incoming --
		if( GetLocale() == "deDE" ) then
			if(element == "Erd") then
				element = "Erde"; -- "Werkzeuge: Erdtotem", rename Erd to Erde
			end
		end
		data.element = L:GetReverseTranslation(element);
		if(not data.element) then
			self:Debug("Localization bug incoming");
		end
	end
	-- End Element --
	text = YataTT:GetLine(5,nil);
	-- Start HP --
	local _, _, health = YataTT:Find("(%d+) "..L["health"],5,5,false,true);
	if(not health) then
		_, _, health = YataTT:Find("(%d+) "..L["points"].." "..L["health"],5,5,false,true);
	end
	data.health = health;
	-- End HP --
	-- Start Duration --     
	     local matches = {};
	     for num, regex in ipairs(Yata_TotemTiming) do
		     for k in string.gmatch(text,"(%d+) "..regex) do
	       		if(num == 1) then
	       			table.insert(matches,tonumber(k*60));
	       		else 	table.insert(matches,tonumber(k));
	       		end
		     end
	     end
	     table.sort(matches);
	     data.duration = matches[table.getn(matches)];
	-- End Duration --
	-- Start Cooldown -- 
	local matches = {};
	text = YataTT:GetLine(3,true);
	if(text) then
	     for num, regex in ipairs(Yata_TotemTiming) do
		     for k in string.gmatch(text,"(%d+) "..regex) do
	       		if(num == 1) then
	       			table.insert(matches,tonumber(k*60));
	       		else 	table.insert(matches,tonumber(k));
	       		end
		     end
	     end
	     table.sort(matches);
	     data.Cooldown = matches[table.getn(matches)];
	end
	-- End Cooldown --
	return data;
end
function Yata:OutputTotemData()
	for k,v in ipairs(self.TotemData) do
		for l,w in ipairs(self.TotemData[v]) do
			self:Print(v.." ".."(Rank "..l.."), HP: "..w.Health..", Duration: "..w.Duration..", Element: "..w.Element..", Cooldown: "..w.Cooldown..", Mana: "..w.Mana);
		end
	end
end